/* globals PRODUCTION */
import "babel-polyfill"
import "dotenv/config"
import express from "express"
import compression from "compression"
import graphqlHTTP from "express-graphql"
import { schema, rootValue } from "./graphql"
import https from "https"
import http from "http"
import sql from "mssql"
import moment from "moment"
import IO from "./io"
import cucm from "./db/helpers/CUCM/"
import fs from "fs"
import cors from "cors"
import { ServePicture, Prevention } from "./extensions"
import runScheduledTasks from "./scheduledTasks"
import generateFlags from "./jobs/generateFlags"

moment.locale( "pt" )

global.PRODUCTION = process.env.PRODUCTION
global.MASTER_KEY = process.env.MASTER_KEY
global.DYNAMICS = process.env.DYNAMICS
global.HOSTNAME = process.env.HOSTNAME
global.ERP = process.env.ERP
global.UPLOAD_PATH = process.env.UPLOAD_PATH
global.HOSTNAME = process.env.HOSTNAME
global.GRAPHQL_PORT = process.env.GRAPHQL_PORT

const privateKey = PRODUCTION ? fs.readFileSync( `${ process.env.CERTIFICATE_PATH }/server.key`, "utf8" ) : null
const certificate = PRODUCTION ? fs.readFileSync( `${ process.env.CERTIFICATE_PATH }/server.cer`, "utf8" ) : null

const credentials = PRODUCTION ? { key: privateKey, cert: certificate } : null
const config = {
  user: process.env.PRIMAVERA_USER,
  password: process.env.PRIMAVERA_PASS,
  encrypt: true,
  server: process.env.PRIMAVERA_SERVER + "\\" + process.env.PRIMAVERA_INSTANCE,
  database: process.env.PRIMAVERA_DATABASE,
  connectionTimeout: 300000,
  requestTimeout: 300000,
  pool: {
    idleTimeoutMillis: 300000,
    max: 100,
  },
}

const init = async ( params ) => {
  const { onlySetupSql } = ( params || {} )
  console.debug( "BEGIN API INITIALIZATION:" )
  console.debug( "SERVER HOSTNAME: ", global.HOSTNAME )
  console.debug( "UPLOAD PATH: ", global.UPLOAD_PATH )
  console.debug( "CRM PROXY: ", global.DYNAMICS )
  console.debug( "ERP PROXY: ", global.ERP )
  console.debug( "GRAPHQL PORT: ", global.GRAPHQL_PORT )
  console.debug( "MAILER MODE: ", process.env.MAIL_DEVELOPER === "false" ? "Normal" : "Developer" )

  await sql.connect( config )
  if ( onlySetupSql ) return
  console.debug( "MSSQL CONNECTION ESTABLISHED" )

  const app = express()
  Prevention( app )
  ServePicture( app )
  app.use( cors() )

  app.use( "/graphqli", ( req, res, next ) =>
    graphqlHTTP( {
      rootValue: rootValue( { io, req, cucm } ),
      schema,
      graphiql: true,
      limit: 99999999,
    } )( req, res, next )
  )
  app.use( "/graphql", ( req, res, next ) =>
    graphqlHTTP( {
      rootValue: rootValue( { io, req, cucm } ),
      schema,
      graphiql: false,
      limit: 99999999,
    } )( req, res, next )
  )
  console.debug( `GRAPHQL API INITIALIZED on port ${ global.GRAPHQL_PORT }` )
  await generateFlags()

  app.use( compression( {
    level: 7,
    filter: () => true,
  } ) )

  const server = credentials ? https.createServer( credentials, app ) : http.createServer( app )
  server.listen( global.GRAPHQL_PORT )

  const io = await IO( server )
  runScheduledTasks()
}

!process.env.DRYRUN && init()

export default init
